@@ -3,21 +3,19 @@
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 import os
+import pathlib
 import warnings
 from functools import lru_cache
 from urllib.parse import urlparse
 
-from jupyter_server.extension.handler import (
-    ExtensionHandlerJinjaMixin,
-    ExtensionHandlerMixin,
-)
+from jupyter_server.base.handlers import FileFindHandler, JupyterHandler
+from jupyter_server.extension.handler import ExtensionHandlerJinjaMixin, ExtensionHandlerMixin
+from jupyter_server.utils import url_path_join as ujoin
 from tornado import template, web
 
 from .config import LabConfig, get_page_config, recursive_update
 from .licenses_handler import LicensesHandler, LicensesManager
 from .listings_handler import ListingsHandler, fetch_listings
-from .server import FileFindHandler, JupyterHandler
-from .server import url_path_join as ujoin
 from .settings_handler import SettingsHandler
 from .themes_handler import ThemesHandler
 from .translations_handler import TranslationsHandler
@@ -66,7 +64,7 @@ class LabHandler(ExtensionHandlerJinjaMixin, ExtensionHandlerMixin, JupyterHandl
     @lru_cache()  # noqa
     def get_page_config(self):
         """Construct the page config object"""
-        self.application.store_id = getattr(self.application, "store_id", 0)
+        self.application.store_id = getattr(self.application, "store_id", 0)  # type:ignore
         config = LabConfig()
         app = self.extensionapp
         settings_dir = app.app_settings_dir
@@ -84,21 +82,28 @@ class LabHandler(ExtensionHandlerJinjaMixin, ExtensionHandlerMixin, JupyterHandl
         page_config.setdefault("terminalsAvailable", terminals)
         page_config.setdefault("ignorePlugins", [])
         page_config.setdefault("serverRoot", server_root)
-        page_config["store_id"] = self.application.store_id
+        page_config["store_id"] = self.application.store_id  # type:ignore
 
         server_root = os.path.normpath(os.path.expanduser(server_root))
+        preferred_path = ""
         try:
-            # Remove the server_root from pref dir
-            if self.serverapp.preferred_dir != server_root:
-                page_config["preferredPath"] = "/" + os.path.relpath(
-                    self.serverapp.preferred_dir, server_root
-                )
-            else:
-                page_config["preferredPath"] = "/"
+            preferred_path = self.serverapp.contents_manager.preferred_dir
         except Exception:
-            page_config["preferredPath"] = "/"
+            # FIXME: Remove fallback once CM.preferred_dir is ubiquitous.
+            try:
+                # Remove the server_root from app pref dir
+                if self.serverapp.preferred_dir and self.serverapp.preferred_dir != server_root:
+                    preferred_path = (
+                        pathlib.Path(self.serverapp.preferred_dir)
+                        .relative_to(server_root)
+                        .as_posix()
+                    )
+            except Exception:  # noqa S110
+                pass
+        # JupyterLab relies on an unset/default path being "/"
+        page_config["preferredPath"] = preferred_path or "/"
 
-        self.application.store_id += 1
+        self.application.store_id += 1  # type:ignore
 
         mathjax_config = self.settings.get("mathjax_config", "TeX-AMS_HTML-full,Safe")
         # TODO Remove CDN usage.
@@ -160,14 +165,17 @@ class LabHandler(ExtensionHandlerJinjaMixin, ExtensionHandlerMixin, JupyterHandl
 
 
 class NotFoundHandler(LabHandler):
+    """A handler for page not found."""
+
     @lru_cache()  # noqa
     def get_page_config(self):
+        """Get the page config."""
         page_config = super().get_page_config()
         page_config["notFoundUrl"] = self.request.path
         return page_config
 
 
-def add_handlers(handlers, extension_app):
+def add_handlers(handlers, extension_app):  # noqa
     """Add the appropriate handlers to the web app."""
     # Normalize directories.
     for name in LabConfig.class_trait_names():
@@ -238,7 +246,6 @@ def add_handlers(handlers, extension_app):
 
     # Handle saved workspaces.
     if extension_app.workspaces_dir:
-
         workspaces_config = {"manager": WorkspacesManager(extension_app.workspaces_dir)}
 
         # Handle requests for the list of workspaces. Make slash optional.
@@ -283,12 +290,13 @@ def add_handlers(handlers, extension_app):
     ):
         from tornado import ioloop
 
+        callback_time = ListingsHandler.listings_refresh_seconds * 1000
         ListingsHandler.pc = ioloop.PeriodicCallback(
-            lambda: fetch_listings(None),
-            callback_time=ListingsHandler.listings_refresh_seconds * 1000,
+            lambda: fetch_listings(None),  # type:ignore
+            callback_time=callback_time,
             jitter=0.1,
         )
-        ListingsHandler.pc.start()
+        ListingsHandler.pc.start()  # type:ignore
 
     handlers.append((listings_path, ListingsHandler, {}))
 
@@ -322,7 +330,7 @@ def add_handlers(handlers, extension_app):
     handlers.append((fallthrough_url, NotFoundHandler))
 
 
-def _camelCase(base):
+def _camelCase(base):  # noqa
     """Convert a string to camelCase.
     https://stackoverflow.com/a/20744956
     """
